@@ -359,11 +359,12 @@ int docmd_rtn(arg_struct *arg) {
     if (program_running()) {
         int newprgm;
         int4 newpc;
-        pop_rtn_addr(&newprgm, &newpc);
+        bool stop;
+        pop_rtn_addr(&newprgm, &newpc, &stop);
         if (newprgm == -3)
-            return return_to_integ(0);
+            return return_to_integ(0, stop);
         else if (newprgm == -2)
-            return return_to_solve(0);
+            return return_to_solve(0, stop);
         else if (newprgm == -1) {
             if (pc >= prgms[current_prgm].size)
                 /* It's an END; go to line 0 */
@@ -372,7 +373,7 @@ int docmd_rtn(arg_struct *arg) {
         } else {
             current_prgm = newprgm;
             pc = newpc;
-            return ERR_NONE;
+            return stop ? ERR_STOP : ERR_NONE;
         }
     } else {
         clear_all_rtns();
@@ -467,10 +468,10 @@ int view_helper(arg_struct *arg, int print) {
 
     if (print && (flags.f.printer_enable || !program_running())) {
         if (flags.f.printer_exists) {
-	    shell_annunciators(-1, -1, 1, -1, -1, -1);
+            shell_annunciators(-1, -1, 1, -1, -1, -1);
             print_wide(buf, part2, buf + part2, bufptr - part2);
-	    shell_annunciators(-1, -1, 0, -1, -1, -1);
-	} else
+            shell_annunciators(-1, -1, 0, -1, -1, -1);
+        } else
             return ERR_STOP;
     }
     return ERR_NONE;
@@ -525,12 +526,17 @@ int docmd_aview(arg_struct *arg) {
 
 int docmd_xeq(arg_struct *arg) {
     if (program_running()) {
-        int oldprgm = current_prgm;
-        int4 oldpc = pc;
-        int error = docmd_gto(arg);
-        if (error != ERR_NONE)
-            return error;
-        return push_rtn_addr(oldprgm, oldpc);
+        int err = push_rtn_addr(current_prgm, pc);
+        if (err != ERR_NONE)
+            return err;
+        err = docmd_gto(arg);
+        if (err != ERR_NONE) {
+            int dummy1;
+            int4 dummy2;
+            bool dummy3;
+            pop_rtn_addr(&dummy1, &dummy2, &dummy3);
+        }
+        return err;
     } else {
         int err = docmd_gto(arg);
         if (err != ERR_NONE)
@@ -1457,12 +1463,14 @@ static int prusr_worker(int interrupted) {
             prusr_index = 0;
             goto state1;
         }
-        llen = 0;
-        string2buf(lbuf, 8, &llen, vars[prusr_index].name,
-                                   vars[prusr_index].length);
-        char2buf(lbuf, 8, &llen, '=');
-        rlen = vartype2string(vars[prusr_index].value, rbuf, 100);
-        print_wide(lbuf, llen, rbuf, rlen);
+        if (!vars[prusr_index].hidden) {
+            llen = 0;
+            string2buf(lbuf, 8, &llen, vars[prusr_index].name,
+                                       vars[prusr_index].length);
+            char2buf(lbuf, 8, &llen, '=');
+            rlen = vartype2string(vars[prusr_index].value, rbuf, 100);
+            print_wide(lbuf, llen, rbuf, rlen);
+        }
         prusr_index--;
     } else {
         char buf[13];
