@@ -242,7 +242,11 @@ static int call_solve_fn(int which, int state) {
         v = new_real(x);
         if (v == NULL)
             return ERR_INSUFFICIENT_MEMORY;
-        store_var(solve.var_name, solve.var_length, v);
+        err = store_var(solve.var_name, solve.var_length, v);
+        if (err != ERR_NONE) {
+            free_vartype(v);
+            return err;
+        }
     } else
         ((vartype_real *) v)->x = x;
     solve.which = which;
@@ -256,8 +260,13 @@ static int call_solve_fn(int which, int state) {
         free_vartype(v);
         return err;
     }
-    push_rtn_addr(-2, 0);
-    return ERR_RUN;
+    err = push_rtn_addr(-2, 0);
+    if (err != ERR_NONE) {
+        current_prgm = solve.prev_prgm;
+        pc = solve.prev_pc;
+        return err;
+    } else
+        return ERR_RUN;
 }
 
 int start_solve(const char *name, int length, phloat x1, phloat x2) {
@@ -291,7 +300,7 @@ int start_solve(const char *name, int length, phloat x1, phloat x2) {
     }
     solve.last_disp_time = 0;
     solve.toggle = 1;
-    solve.keep_running = program_running();
+    solve.keep_running = !should_i_stop_at_this_level() && program_running();
     return call_solve_fn(1, 1);
 }
 
@@ -415,10 +424,13 @@ static void printout(const phloat& p, const char* s) {
 }
 #endif
 
-int return_to_solve(int failure) {
+int return_to_solve(int failure, bool stop) {
     phloat f, slope, s, xnew, prev_f = solve.curr_f;
     uint4 now_time;
 
+    if (stop)
+        solve.keep_running = 0;
+
     if (solve.state == 0)
         return ERR_INTERNAL_ERROR;
     if (!failure) {
@@ -849,7 +861,11 @@ static int call_integ_fn() {
         v = new_real(x);
         if (v == NULL)
             return ERR_INSUFFICIENT_MEMORY;
-        store_var(integ.var_name, integ.var_length, v);
+        err = store_var(integ.var_name, integ.var_length, v);
+        if (err != ERR_NONE) {
+            free_vartype(v);
+            return err;
+        }
     } else
         ((vartype_real *) v)->x = x;
     arg.type = ARGTYPE_STR;
@@ -861,8 +877,13 @@ static int call_integ_fn() {
         free_vartype(v);
         return err;
     }
-    push_rtn_addr(-3, 0);
-    return ERR_RUN;
+    err = push_rtn_addr(-3, 0);
+    if (err != ERR_NONE) {
+        current_prgm = integ.prev_prgm;
+        pc = integ.prev_pc;
+        return err;
+    } else
+        return ERR_RUN;
 }
 
 int start_integ(const char *name, int length) {
@@ -913,7 +934,7 @@ int start_integ(const char *name, int length) {
     integ.k = 1;
     integ.prev_res = 0;
 
-    integ.keep_running = program_running();
+    integ.keep_running = !should_i_stop_at_this_level() && program_running();
     if (!integ.keep_running) {
         clear_row(0);
         draw_string(0, 0, "Integrating", 11);
@@ -921,7 +942,7 @@ int start_integ(const char *name, int length) {
         flags.f.message = 1;
         flags.f.two_line_message = 0;
     }
-    return return_to_integ(0);
+    return return_to_integ(0, false);
 }
 
 static int finish_integ() {
@@ -966,7 +987,10 @@ static int finish_integ() {
  * which prevents endpoint evaluation and causes non-uniform sampling.
  */
 
-int return_to_integ(int failure) {
+int return_to_integ(int failure, bool stop) {
+    if (stop)
+        integ.keep_running = 0;
+    
     switch (integ.state) {
     case 0:
         return ERR_INTERNAL_ERROR;
