@@ -37,7 +37,7 @@ void skin_repaint(NSRect *rect);
 void skin_update_annunciator(int which, int state);
 void skin_find_key(int x, int y, bool cshift, int *key, int *code);
 int skin_find_skey(int ckey);
-unsigned char *skin_find_macro(int ckey);
+unsigned char *skin_find_macro(int ckey, bool *is_name);
 unsigned char *skin_keymap_lookup(unsigned short keychar, bool printable,
                   bool ctrl, bool alt, bool shift, bool cshift,
                   bool *exact);
